precision highp float;
varying highp vec2 textureCoordinate;

uniform sampler2D inputImageTexture;
uniform mediump float hueAdjust;
const highp vec4 kRGBToYPrime = vec4 (0.299, 0.587, 0.114, 0.0);
const highp vec4 kRGBToI = vec4 (0.595716, -0.274453, -0.321263, 0.0);
const highp vec4 kRGBToQ = vec4 (0.211456, -0.522591, 0.31135, 0.0);

const highp vec4 kYIQToR = vec4 (1.0, 0.9563, 0.6210, 0.0);
const highp vec4 kYIQToG = vec4 (1.0, -0.2721, -0.6474, 0.0);
const highp vec4 kYIQToB = vec4 (1.0, -1.1070, 1.7046, 0.0);
highp vec3 rgb2hsl(highp vec3 rgb) {
    highp float r = rgb.x;//0
    highp float g = rgb.y;//0
    highp float b = rgb.z;//255
    highp float max = max(r, max(g, b));//255
    highp float min = min(r, min(g, b));//0
    highp float h;
    highp float s;
    highp float l = (max + min) / 2.0;//127.5
    if (max == min) {
        h = 0.0;
        s = 0.0;
    } else {
        highp float d = max - min;//255
        s = l > 0.5 ? d / (2.0 - max - min) : d / (max + min);//-1.0
        if (max == r) {
            h = ((g - b) / d + (g < b ? 6.0 : 0.0)) / 6.0;
        } else if (max == g) {
            h = ((b - r) / d + 2.0) / 6.0;
        } else if (max == b){
            h = ((r - g) / d + 4.0) / 6.0;//
        }
    }
    return vec3(h, s, l);
}
uniform highp float Max;
uniform highp float Min;
void main (){
    // Sample the input pixel  对输入像素采样
    highp vec4 rgba = texture2D(inputImageTexture, textureCoordinate);
    highp vec3 hsl = rgb2hsl(rgba.xyz);
    vec3 col = texture2D(inputImageTexture, textureCoordinate).rgb;

    if (hsl.x<=Max&&hsl.x>=Min){
        // Convert to YIQ  转换到YIQ
        highp float YPrime = dot (rgba, kRGBToYPrime);
        highp float I = dot (rgba, kRGBToI);
        highp float Q = dot (rgba, kRGBToQ);

        // Calculate the hue and chroma   计算色调和色度
        highp float hue = atan (Q, I);
        highp float chroma = sqrt (I * I + Q * Q);

        // Make the user's adjustments  进行用户的调整
        hue += (-hueAdjust);//why negative rotation?  为什么负旋转?

        // Convert back to YIQ  转换回YIQ
        Q = chroma * sin (hue);
        I = chroma * cos (hue);

        // Convert back to RGB 转换回RGB
        highp vec4 yIQ = vec4 (YPrime, I, Q, 0.0);
        rgba.r = dot (yIQ, kYIQToR);
        rgba.g = dot (yIQ, kYIQToG);
        rgba.b = dot (yIQ, kYIQToB);
        // Save the result 保存结果
    }
    gl_FragColor = rgba;
}