varying highp vec2 textureCoordinate;

uniform sampler2D inputImageTexture;
uniform lowp float saturation;

// Values from \Graphics Shaders: Theory and Practice\ by Bailey and Cunningham
const mediump vec3 luminanceWeighting = vec3(0.2125, 0.7154, 0.0721);

highp vec3 rgb2hsl(highp vec3 rgb) {
    highp float r = rgb.x;//0
    highp float g = rgb.y;//0
    highp float b = rgb.z;//255
    highp float max = max(r, max(g, b));//255
    highp float min = min(r, min(g, b));//0
    highp float h;
    highp float s;
    highp float l = (max + min) / 2.0;//127.5
    if (max == min) {
        h = 0.0;
        s = 0.0;
    } else {
        highp float d = max - min;//255
        s = l > 0.5 ? d / (2.0 - max - min) : d / (max + min);//-1.0
        if (max == r) {
            h = ((g - b) / d + (g < b ? 6.0 : 0.0)) / 6.0;
        } else if (max == g) {
            h = ((b - r) / d + 2.0) / 6.0;
        } else if (max == b){
            h = ((r - g) / d + 4.0) / 6.0;//
        }
    }
    return vec3(h, s, l);
}
void main(){
    lowp vec4 rgba = texture2D(inputImageTexture, textureCoordinate);
    highp vec3 hsl = rgb2hsl(rgba.xyz);
    if (hsl.x<=0.08737864f||hsl.x>=0.7423246f){
        lowp float luminance = dot(rgba.rgb, luminanceWeighting);
        lowp vec3 greyScaleColor = vec3(luminance);

        gl_FragColor = vec4(mix(greyScaleColor, rgba.rgb, saturation), rgba.w);
    } else {
        gl_FragColor = rgba;
    }
}